@@ -26,15 +26,19 @@ posedge -- callable to model a rising edge on a signal in a yield statement
 negedge -- callable to model a falling edge on a signal in a yield statement
 
 """
+from __future__ import absolute_import
+from __future__ import print_function
 
 from inspect import currentframe, getouterframes
 from copy import copy, deepcopy
 import operator
 
+from myhdl._compat import integer_types, long
 from myhdl import _simulator as sim
 from myhdl._simulator import _signals, _siglist, _futureEvents, now
 from myhdl._intbv import intbv
 from myhdl._bin import bin
+
 # from myhdl._enum import EnumItemType
 
 _schedule = _futureEvents.append
@@ -134,8 +138,8 @@ class _Signal(object):
             self._setNextVal = self._setNextBool
             self._printVcd = self._printVcdBit
             self._nrbits = 1
-        elif isinstance(val, (int, long)):
-            self._type = (int, long)
+        elif isinstance(val, integer_types):
+            self._type = integer_types
             self._setNextVal = self._setNextInt
         elif isinstance(val, intbv):
             self._type = intbv
@@ -189,7 +193,7 @@ class _Signal(object):
                 self._val = None
             elif isinstance(val, intbv):
                 self._val._val = next._val
-            elif isinstance(val, (int, long, EnumItemType)):
+            elif isinstance(val, (integer_types, EnumItemType)):
                 self._val = next
             else:
                 self._val = deepcopy(next)
@@ -200,58 +204,65 @@ class _Signal(object):
             return []
 
     # support for the 'val' attribute
-    def _get_val(self):
+    @property
+    def val(self):
         return self._val
-    val = property(_get_val, None, None, "'val' access methods")
 
     # support for the 'next' attribute
-    def _get_next(self):
+    @property
+    def next(self):
 #        if self._next is self._val:
 #            self._next = deepcopy(self._val)
         _siglist.append(self)
         return self._next
-    def _set_next(self, val):
+
+    @next.setter
+    def next(self, val):
         if isinstance(val, _Signal):
             val = val._val
         self._setNextVal(val)
         _siglist.append(self)
-    next = property(_get_next, _set_next, None, "'next' access methods")
 
     # support for the 'posedge' attribute
-    def _get_posedge(self):
+    @property
+    def posedge(self):
         return self._posedgeWaiters
-    posedge = property(_get_posedge, None, None, "'posedge' access methods")
                        
     # support for the 'negedge' attribute
-    def _get_negedge(self):
+    @property
+    def negedge(self):
         return self._negedgeWaiters
-    negedge = property(_get_negedge, None, None, "'negedge' access methods")
     
     # support for the 'min' and 'max' attribute
-    def _get_max(self):
+    @property
+    def max(self):
         return self._max
-    max = property(_get_max, None)
-    def _get_min(self):
+
+    @property
+    def min(self):
         return self._min
-    min = property(_get_min, None)
 
     # support for the 'driven' attribute
-    def _get_driven(self):
+    @property
+    def driven(self):
         return self._driven
-    def _set_driven(self, val):
+
+    @driven.setter
+    def driven(self, val):
         if not val  in ("reg", "wire", True):
             raise ValueError('Expected value "reg", "wire", or True, got "%s"' % val)
         self._driven = val
-    driven = property(_get_driven, _set_driven, None, "'driven' access methods")
     
     # support for the 'read' attribute
-    def _get_read(self):
+    @property
+    def read(self):
         return self._read
-    def _set_read(self, val):
+
+    @read.setter
+    def read(self, val):
         if not val in (True, ):
             raise ValueError('Expected value True, got "%s"' % val)
         self._markRead()
-    read = property(_get_read, _set_read, None, "'read' access methods")
 
     def _markRead(self):
         self._read = True
@@ -262,22 +273,24 @@ class _Signal(object):
 
     # set next methods
     def _setNextBool(self, val):
+        if isinstance(val, intbv):
+            val = val._val
         if not val in (0, 1):
             raise ValueError("Expected boolean value, got %s (%s)" % (repr(val), type(val)))
         self._next = val
 
     def _setNextInt(self, val):
-        if not isinstance(val, (int, long, intbv)):
+        if isinstance(val, intbv):
+            val = val._val
+        elif not isinstance(val, (integer_types, intbv)):
             raise TypeError("Expected int or intbv, got %s" % type(val))
         self._next = val
 
     def _setNextIntbv(self, val):
         if isinstance(val, intbv):
             val = val._val
-        elif not isinstance(val, (int, long)):
+        elif not isinstance(val, integer_types):
             raise TypeError("Expected int or intbv, got %s" % type(val))
-#        if self._next is self._val:
-#            self._next = type(self._val)(self._val)
         self._next._val = val
         self._next._handleBounds()
 
@@ -293,16 +306,25 @@ class _Signal(object):
 
     # vcd print methods
     def _printVcdStr(self):
-        print >> sim._tf, "s%s %s" % (str(self._val), self._code)
+        print("s%s %s" % (str(self._val), self._code), file=sim._tf)
         
     def _printVcdHex(self):
-        print >> sim._tf, "s%s %s" % (hex(self._val), self._code)
+        if self._val is None:
+            print("sz %s" % self._code, file=sim._tf)
+        else:
+            print("s%s %s" % (hex(self._val), self._code), file=sim._tf)
 
     def _printVcdBit(self):
-        print >> sim._tf, "%d%s" % (self._val, self._code)
+        if self._val is None:
+            print("z%s" % self._code, file=sim._tf)
+        else:
+            print("%d%s" % (self._val, self._code), file=sim._tf)
 
     def _printVcdVec(self):
-        print >> sim._tf, "b%s %s" % (bin(self._val, self._nrbits), self._code)
+        if self._val is None:
+            print("b%s %s" % ('z'*self._nrbits, self._code), file=sim._tf)
+        else:
+            print("b%s %s" % (bin(self._val, self._nrbits), self._code), file=sim._tf)
 
     ### use call interface for shadow signals ###
     def __call__(self, left, right=None):
@@ -317,11 +339,10 @@ class _Signal(object):
         raise TypeError("Signals are unhashable")
         
     
-    def __nonzero__(self):
-        if self._val:
-            return 1
-        else:
-            return 0
+    def __bool__(self):
+        return bool(self._val)
+
+    __nonzero__ = __bool__
 
     # length
     def __len__(self):
@@ -359,21 +380,13 @@ class _Signal(object):
     def __rmul__(self, other):
         return other * self._val
 
-    def __div__(self, other):
+    def __truediv__(self, other):
         if isinstance(other, _Signal):
             return self._val / other._val
         else:
             return self._val / other
-    def __rdiv__(self, other):
-        return other / self._val
-    
-    def __truediv__(self, other):
-        if isinstance(other, _Signal):
-            return operator.truediv(self._val, other._val)
-        else:
-            return operator.truediv(self._val, other)
     def __rtruediv__(self, other):
-        return operator.truediv(other, self._val)
+        return other / self._val
     
     def __floordiv__(self, other):
         if isinstance(other, _Signal):
@@ -508,14 +521,16 @@ class _Signal(object):
 
     # augmented assignment not supported
     def _augm(self):
-        raise TypeError, "Signal object doesn't support augmented assignment"
+        raise TypeError("Signal object doesn't support augmented assignment")
 
-    __iadd__ = __isub__ = __idiv__ = __imul__ = __ipow__ = __imod__ = _augm
+    __iadd__ = __isub__ = __imul__ = __ipow__ = __imod__ = _augm
     __ior__ = __iand__ = __ixor__ = __irshift__ = __ilshift__ = _augm
+    __itruediv__ = __ifloordiv__ = _augm
+
 
     # index and slice assignment not supported
     def __setitem__(self, key, val):
-        raise TypeError, "Signal object doesn't support item/slice assignment"
+        raise TypeError("Signal object doesn't support item/slice assignment")
 
 
     # continues assignment support
@@ -583,12 +598,14 @@ class _DelayedSignal(_Signal):
         else:
             return []
 
-   # support for the 'delay' attribute
-    def _get_delay(self):
-         return self._delay
-    def _set_delay(self, delay):
-         self._delay = delay
-    delay = property(_get_delay, _set_delay, None, "'delay' access methods")
+    # support for the 'delay' attribute
+    @property
+    def delay(self):
+        return self._delay
+
+    @delay.setter
+    def delay(self, delay):
+        self._delay = delay
 
         
 class _SignalWrap(object):
