@@ -40,7 +40,7 @@ bin -- returns a binary string representation.
 concat -- function to concat ints, bitstrings, bools, intbvs, Signals
        -- returns an intbv
 instances -- function that returns all instances defined in a function
-always -- 
+always --
 always_comb -- decorator that returns an input-sensitive generator
 always_seq --
 ResetSignal --
@@ -49,8 +49,10 @@ traceSignals -- function that enables signal tracing in a VCD file
 toVerilog -- function that converts a design to Verilog
 
 """
+from __future__ import absolute_import
+from __future__ import print_function
 
-__version__ = "0.8.1"
+__version__ = "0.9.0"
 
 import sys
 import warnings
@@ -106,36 +108,36 @@ class ToVHDLWarning(ConversionWarning):
 # warnings.filterwarnings('always', r".*", ToVerilogWarning)
 
 def showwarning(message, category, filename, lineno, *args):
-    print >> sys.stderr, "** %s: %s" % (category.__name__, message)
+    print("** %s: %s" % (category.__name__, message), file=sys.stderr)
 
 warnings.showwarning = showwarning
 
 
-from _bin import bin
-from _concat import concat
-from _intbv import intbv
-from _modbv import modbv
-from _join import join
-from _Signal import posedge, negedge, Signal, SignalType
-from _ShadowSignal import ConcatSignal
-from _ShadowSignal import TristateSignal
-from _simulator import now
-from _delay import delay
-from _Cosimulation import Cosimulation
-from _Simulation import Simulation
-from _misc import instances, downrange
-from _always_comb import always_comb
-from _always_seq import always_seq, ResetSignal
-from _always import always
-from _instance import instance
-from _enum import enum, EnumType, EnumItemType
-from _traceSignals import traceSignals
+from ._bin import bin
+from ._concat import concat
+from ._intbv import intbv
+from ._modbv import modbv
+from ._join import join
+from ._Signal import posedge, negedge, Signal, SignalType
+from ._ShadowSignal import ConcatSignal
+from ._ShadowSignal import TristateSignal
+from ._simulator import now
+from ._delay import delay
+from ._Cosimulation import Cosimulation
+from ._Simulation import Simulation
+from ._misc import instances, downrange
+from ._always_comb import always_comb
+from ._always_seq import always_seq, ResetSignal
+from ._always import always
+from ._instance import instance
+from ._enum import enum, EnumType, EnumItemType
+from ._traceSignals import traceSignals
 
 from myhdl import conversion
-from conversion import toVerilog
-from conversion import toVHDL
+from .conversion import toVerilog
+from .conversion import toVHDL
 
-from _tristate import Tristate
+from ._tristate import Tristate
 
 
 __all__ = ["bin",
@@ -168,7 +170,7 @@ __all__ = ["bin",
            "toVerilog",
            "toVHDL",
            "conversion",
-           "Tristate"           
+           "Tristate"
            ]
 
 
