@@ -18,6 +18,7 @@
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 
 """ Module with the always_comb function. """
+from __future__ import absolute_import
 
 import sys
 import inspect
@@ -31,6 +32,7 @@ from myhdl._util import _isGenFunc, _dedent
 from myhdl._cell_deref import _cell_deref
 from myhdl._Waiter import _Waiter, _SignalWaiter, _SignalTupleWaiter
 from myhdl._instance import _Instantiator
+from myhdl._resolverefs import _AttrRefTransformer
 
 class _error:
     pass
@@ -40,22 +42,22 @@ _error.Scope = "always_comb argument should be a local function"
 _error.SignalAsInout = "signal (%s) used as inout in always_comb function argument"
 _error.EmbeddedFunction = "embedded functions in always_comb function argument not supported"
 _error.EmptySensitivityList= "sensitivity list is empty"
-    
+
 def always_comb(func):
     if not isinstance( func, FunctionType):
         raise AlwaysCombError(_error.ArgType)
     if _isGenFunc(func):
         raise AlwaysCombError(_error.ArgType)
-    if func.func_code.co_argcount > 0:
+    if func.__code__.co_argcount > 0:
         raise AlwaysCombError(_error.NrOfArgs)
-    varnames = func.func_code.co_varnames
+    varnames = func.__code__.co_varnames
     symdict = {}
-    for n, v in func.func_globals.items():
+    for n, v in func.__globals__.items():
         if n not in varnames:
             symdict[n] = v
     # handle free variables
-    if func.func_code.co_freevars:
-        for n, c in zip(func.func_code.co_freevars, func.func_closure):
+    if func.__code__.co_freevars:
+        for n, c in zip(func.__code__.co_freevars, func.__closure__):
             try:
                 obj = _cell_deref(c)
                 symdict[n] = obj
@@ -63,7 +65,7 @@ def always_comb(func):
                 raise NameError(n)
     c = _AlwaysComb(func, symdict)
     return c
-   
+
 
 INPUT, OUTPUT, INOUT = range(3)
 
@@ -115,7 +117,7 @@ class _SigNameVisitor(ast.NodeVisitor):
                 raise AlwaysCombError(_error.SignalAsInout % id)
             else:
                 raise AssertionError("bug in always_comb")
-            
+
     def visit_Assign(self, node):
         self.context = OUTPUT
         for n in node.targets:
@@ -146,7 +148,7 @@ class _SigNameVisitor(ast.NodeVisitor):
         self.visit(node.target)
         self.context = INPUT
         self.visit(node.value)
-        
+
     def visit_ClassDef(self, node):
         pass # skip
 
@@ -156,7 +158,7 @@ class _SigNameVisitor(ast.NodeVisitor):
     def visit_Print(self, node):
         pass # skip
 
-         
+
 
 class _AlwaysComb(_Instantiator):
 
@@ -196,7 +198,9 @@ class _AlwaysComb(_Instantiator):
         s = _dedent(s)
         tree = ast.parse(s)
         # print ast.dump(tree)
-        v = _SigNameVisitor(symdict)
+        v = _AttrRefTransformer(self)
+        v.visit(tree)
+        v = _SigNameVisitor(self.symdict)
         v.visit(tree)
         self.inputs = v.inputs
         self.outputs = v.outputs
@@ -227,7 +231,7 @@ class _AlwaysComb(_Instantiator):
         while 1:
             func()
             yield senslist
- 
+
 
 
 
