@@ -18,6 +18,7 @@
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 
 """ Module with the always_seq decorator. """
+from __future__ import absolute_import
 
 
 import sys
@@ -32,6 +33,7 @@ from myhdl._delay import delay
 from myhdl._Signal import _Signal, _WaiterList,_isListOfSigs
 from myhdl._Waiter import _Waiter, _EdgeWaiter, _EdgeTupleWaiter
 from myhdl._instance import _Instantiator
+from myhdl._resolverefs import _AttrRefTransformer
 
 # evacuate this later
 AlwaysSeqError = AlwaysError
@@ -48,14 +50,14 @@ _error.EmbeddedFunction = "embedded functions in always_seq function not support
 class ResetSignal(_Signal):
     def __init__(self, val, active, async):
         """ Construct a ResetSignal.
-        
+
         This is to be used in conjunction with the always_seq decorator,
         as the reset argument.
         """
         _Signal.__init__(self, bool(val))
         self.active = bool(active)
         self.async = async
-  
+
 
 
 def always_seq(edge, reset):
@@ -74,11 +76,11 @@ def always_seq(edge, reset):
             raise AlwaysSeqError(_error.ArgType)
         if _isGenFunc(func):
             raise AlwaysSeqError(_error.ArgType)
-        if func.func_code.co_argcount > 0:
+        if func.__code__.co_argcount > 0:
             raise AlwaysSeqError(_error.NrOfArgs)
         return _AlwaysSeq(func, edge, reset)
     return _always_seq_decorator
-        
+
 
 class _AlwaysSeq(_Instantiator):
 
@@ -105,14 +107,14 @@ class _AlwaysSeq(_Instantiator):
 
         # find symdict
         # similar to always_comb, but in class constructor
-        varnames = func.func_code.co_varnames
+        varnames = func.__code__.co_varnames
         symdict = {}
-        for n, v in func.func_globals.items():
+        for n, v in func.__globals__.items():
             if n not in varnames:
                 symdict[n] = v
         # handle free variables
-        if func.func_code.co_freevars:
-            for n, c in zip(func.func_code.co_freevars, func.func_closure):
+        if func.__code__.co_freevars:
+            for n, c in zip(func.__code__.co_freevars, func.__closure__):
                 try:
                     obj = _cell_deref(c)
                     symdict[n] = obj
@@ -125,7 +127,9 @@ class _AlwaysSeq(_Instantiator):
         s = _dedent(s)
         tree = ast.parse(s)
         # print ast.dump(tree)
-        v = _SigNameVisitor(symdict)
+        v = _AttrRefTransformer(self)
+        v.visit(tree)
+        v = _SigNameVisitor(self.symdict)
         v.visit(tree)
         sigregs = self.sigregs = []
         varregs = self.varregs = []
@@ -139,7 +143,7 @@ class _AlwaysSeq(_Instantiator):
                 assert _isListOfSigs(reg)
                 for e in reg:
                     sigregs.append(e)
-                    
+
 
     def reset_sigs(self):
         for s in self.sigregs:
@@ -223,7 +227,7 @@ class _SigNameVisitor(ast.NodeVisitor):
                 raise AlwaysSeqError(_error.SigAugAssign, id)
             else:
                 raise AssertionError("bug in always_seq")
-            
+
     def visit_Assign(self, node):
         self.context = OUTPUT
         for n in node.targets:
@@ -244,7 +248,7 @@ class _SigNameVisitor(ast.NodeVisitor):
         self.visit(node.target)
         self.context = INPUT
         self.visit(node.value)
-        
+
     def visit_ClassDef(self, node):
         pass # skip
 
@@ -254,8 +258,8 @@ class _SigNameVisitor(ast.NodeVisitor):
     def visit_Print(self, node):
         pass # skip
 
-         
 
 
 
- 
+
+
