@@ -20,6 +20,7 @@
 """ MyHDL conversion analysis module.
 
 """
+from __future__ import absolute_import, print_function
 
 import inspect
 # import compiler
@@ -27,7 +28,8 @@ import inspect
 from types import FunctionType, MethodType
 import re
 import ast
-import __builtin__
+from itertools import chain
+from collections import defaultdict
 
 import myhdl
 from myhdl import *
@@ -37,21 +39,24 @@ from myhdl._always_comb import _AlwaysComb
 from myhdl._always_seq import _AlwaysSeq
 from myhdl._always import _Always
 from myhdl.conversion._misc import (_error, _access, _kind,
-                                    _ConversionMixin, _Label, _genUniqueSuffix)
+                                    _ConversionMixin, _Label, _genUniqueSuffix,
+                                    _get_argnames)
 from myhdl._extractHierarchy import _isMem, _getMemInfo, _UserCode
 from myhdl._Signal import _Signal, _WaiterList
-from myhdl._ShadowSignal import _ShadowSignal, _SliceSignal
-from myhdl._util import _isTupleOfInts, _dedent
+from myhdl._ShadowSignal import _ShadowSignal, _SliceSignal, _TristateDriver
+from myhdl._util import _isTupleOfInts, _dedent, _flatten, _makeAST
+from myhdl._resolverefs import _AttrRefTransformer
+from myhdl._compat import builtins, integer_types, PY2
 
 myhdlObjects = myhdl.__dict__.values()
-builtinObjects = __builtin__.__dict__.values()
+builtinObjects = builtins.__dict__.values()
 
 _enumTypeSet = set()
 _constDict = {}
 _extConstDict = {}
 
 
-def _makeName(n, prefixes):
+def _makeName(n, prefixes, namedict):
     # trim empty prefixes
     prefixes = [p for p in prefixes if p]
     if len(prefixes) > 1:
@@ -65,34 +70,29 @@ def _makeName(n, prefixes):
 ##     print name
     return name
 
-def _makeAST(f):
-    s = inspect.getsource(f)
-    s = _dedent(s)
-    tree = ast.parse(s)
-    tree.sourcefile = inspect.getsourcefile(f)
-    tree.lineoffset = inspect.getsourcelines(f)[1]-1
-    return tree
-                     
+
 def _analyzeSigs(hierarchy, hdl='Verilog'):
     curlevel = 0
     siglist = []
     memlist = []
     prefixes = []
+
     open, close = '[', ']'
     if hdl == 'VHDL':
         open, close = '(', ')'
-    
+
     for inst in hierarchy:
         level = inst.level
         name = inst.name
         sigdict = inst.sigdict
         memdict = inst.memdict
+        namedict = dict(chain(sigdict.items(),  memdict.items()))
         delta = curlevel - level
         curlevel = level
         assert(delta >= -1)
         if delta > -1: # same or higher level
-            prefixes = prefixes[:curlevel-1]   
-        # skip processing and prefixing in context without signals    
+            prefixes = prefixes[:curlevel-1]
+        # skip processing and prefixing in context without signals
         if not (sigdict or memdict):
             prefixes.append("")
             continue
@@ -102,7 +102,7 @@ def _analyzeSigs(hierarchy, hdl='Verilog'):
                 continue
             if isinstance(s, _SliceSignal):
                 continue
-            s._name = _makeName(n, prefixes)
+            s._name = _makeName(n, prefixes, namedict)
             if not s._nrbits:
                 raise ConversionError(_error.UndefinedBitWidth, s._name)
             # slice signals
@@ -113,7 +113,7 @@ def _analyzeSigs(hierarchy, hdl='Verilog'):
         for n, m in memdict.items():
             if m.name is not None:
                 continue
-            m.name = _makeName(n, prefixes)
+            m.name = _makeName(n, prefixes, namedict)
             memlist.append(m)
 
     # handle the case where a named signal appears in a list also by giving
@@ -133,10 +133,10 @@ def _analyzeSigs(hierarchy, hdl='Verilog'):
                 raise ConversionError(_error.InconsistentType, s._name)
             if s._nrbits != m.elObj._nrbits:
                 raise ConversionError(_error.InconsistentBitWidth, s._name)
-            
+
     return siglist, memlist
 
-        
+
 
 def _analyzeGens(top, absnames):
     genlist = []
@@ -145,31 +145,21 @@ def _analyzeGens(top, absnames):
             tree = g
         elif isinstance(g, (_AlwaysComb, _AlwaysSeq, _Always)):
             f = g.func
-            s = inspect.getsource(f)
-            s = _dedent(s)
-            tree = ast.parse(s)
-            #print ast.dump(tree)
-            tree.sourcefile  = inspect.getsourcefile(f)
-            tree.lineoffset = inspect.getsourcelines(f)[1]-1
-            tree.symdict = f.func_globals.copy()
+            tree = _makeAST(f)
+            tree.symdict = f.__globals__.copy()
             tree.callstack = []
             # handle free variables
             tree.nonlocaldict = {}
-            if f.func_code.co_freevars:
-                for n, c in zip(f.func_code.co_freevars, f.func_closure):
+            if f.__code__.co_freevars:
+                for n, c in zip(f.__code__.co_freevars, f.__closure__):
                     obj = _cell_deref(c)
-                    if isinstance(g, _AlwaysComb):
-                        if not ( isinstance(obj, (int, long, EnumType,_Signal)) or \
-                                 _isMem(obj) or _isTupleOfInts(obj)
-                               ):
-                            info =  "File %s, line %s: " % (tree.sourcefile, tree.lineoffset)
-                            print type(obj)
-                            raise ConversionError(_error.UnsupportedType, n, info)
                     tree.symdict[n] = obj
                     # currently, only intbv as automatic nonlocals (until Python 3.0)
                     if isinstance(obj, intbv):
                         tree.nonlocaldict[n] = obj
             tree.name = absnames.get(id(g), str(_Label("BLOCK"))).upper()
+            v = _AttrRefTransformer(tree)
+            v.visit(tree)
             v = _FirstPassVisitor(tree)
             v.visit(tree)
             if isinstance(g, _AlwaysComb):
@@ -181,17 +171,14 @@ def _analyzeGens(top, absnames):
             v.visit(tree)
         else: # @instance
             f = g.gen.gi_frame
-            s = inspect.getsource(f)
-            s = _dedent(s)
-            tree = ast.parse(s)
-            # print ast.dump(tree)
-            tree.sourcefile = inspect.getsourcefile(f)
-            tree.lineoffset = inspect.getsourcelines(f)[1]-1
+            tree = _makeAST(f)
             tree.symdict = f.f_globals.copy()
             tree.symdict.update(f.f_locals)
             tree.nonlocaldict = {}
             tree.callstack = []
             tree.name = absnames.get(id(g), str(_Label("BLOCK"))).upper()
+            v = _AttrRefTransformer(tree)
+            v.visit(tree)
             v = _FirstPassVisitor(tree)
             v.visit(tree)
             v = _AnalyzeBlockVisitor(tree)
@@ -203,15 +190,15 @@ def _analyzeGens(top, absnames):
 class _FirstPassVisitor(ast.NodeVisitor, _ConversionMixin):
 
     """First pass visitor.
-    
+
     Prune unsupported contructs, and add some useful attributes.
 
     """
-    
+
     def __init__(self, tree):
         self.tree = tree
         self.toplevel = True
-        
+
     def visit_Tuple(self, node):
         if isinstance(node.ctx, ast.Store):
             self.raiseError(node, _error.NotSupported, "tuple assignment")
@@ -266,13 +253,13 @@ class _FirstPassVisitor(ast.NodeVisitor, _ConversionMixin):
     def visit_TryFinally(self, node):
         self.raiseError(node, _error.NotSupported, "try-finally statement")
 
-        
+
     def visit_Assign(self, node):
         if len(node.targets) > 1:
             self.raiseError(node, _error.NotSupported, "multiple assignments")
         self.visit(node.targets[0])
         self.visit(node.value)
-        
+
     def visit_Call(self, node):
         if node.starargs:
             self.raiseError(node, _error.NotSupported, "extra positional arguments")
@@ -280,19 +267,19 @@ class _FirstPassVisitor(ast.NodeVisitor, _ConversionMixin):
             self.raiseError(node, _error.NotSupported, "extra named arguments")
         # f = eval(_unparse(node.node), self.tree.symdict)
         self.generic_visit(node)
-                
+
     def visit_Compare(self, node):
         if len(node.ops) != 1:
             self.raiseError(node, _error.NotSupported, "chained comparison")
         self.generic_visit(node)
-        
+
     def visit_FunctionDef(self, node):
         if node.args.vararg or node.args.kwarg:
             self.raiseError(node, _error.NotSupported, "extra positional or named arguments")
         if not self.toplevel:
             self.raiseError(node, _error.NotSupported, "embedded function definition")
         self.toplevel = False
-        node.argnames = [arg.id for arg in node.args.args]
+        node.argnames = _get_argnames(node)
         # don't visit decorator lists - they can support more than other calls
         # put official docstrings aside for separate processing
         node.doc = None
@@ -301,7 +288,7 @@ class _FirstPassVisitor(ast.NodeVisitor, _ConversionMixin):
             node.doc = node.body[0].value.s
             node.body = node.body[1:]
         self.visitList(node.body)
-        
+
     def flattenIf(self, node, tests, else_, co):
         """ Flatten if-then-else as in compiler package."""
         if node:
@@ -323,7 +310,7 @@ class _FirstPassVisitor(ast.NodeVisitor, _ConversionMixin):
                     node.ignore = True
                     return # skip
         self.generic_visit(node)
-                
+
         # add fields that match old compiler package
         tests = [(node.test, node.body)]
         else_ = []
@@ -337,7 +324,7 @@ class _FirstPassVisitor(ast.NodeVisitor, _ConversionMixin):
             self.raiseError(node, _error.NotSupported, "printing to a file with >> syntax")
         if not node.nl:
             self.raiseError(node, _error.NotSupported, "printing without newline")
-        
+
 
 
 
@@ -404,11 +391,11 @@ def _getNritems(obj):
     elif isinstance(obj, EnumItemType):
         return len(obj._type)
     else:
-        raise TypeError("Unexpected type")
-        
+        raise TypeError("Unexpected type, missing final \'else:\'?")
+
 
 class _AnalyzeVisitor(ast.NodeVisitor, _ConversionMixin):
-    
+
     def __init__(self, tree):
         tree.sigdict = {}
         tree.vardict = {}
@@ -441,7 +428,7 @@ class _AnalyzeVisitor(ast.NodeVisitor, _ConversionMixin):
             if not hasType(n.obj, bool):
                 self.raiseError(node, _error.NotSupported, "non-boolean argument in logical operator")
         node.obj = bool()
-    
+
     def visit_UnaryOp(self, node):
         self.visit(node.operand)
         op = node.op
@@ -465,7 +452,7 @@ class _AnalyzeVisitor(ast.NodeVisitor, _ConversionMixin):
                 if isinstance(obj, _Signal) and isinstance(obj._init, modbv):
                     if not obj._init._hasFullRange():
                         self.raiseError(node, _error.ModbvRange, n)
-         
+
     def setAttr(self, node):
         if node.attr != 'next':
             self.raiseError(node, _error.NotSupported, "attribute assignment")
@@ -505,7 +492,7 @@ class _AnalyzeVisitor(ast.NodeVisitor, _ConversionMixin):
                 obj._setName(n+suf)
         if node.obj is None: # attribute lookup failed
             self.raiseError(node, _error.UnsupportedAttribute, node.attr)
-        
+
     def visit_Assign(self, node):
         target, value = node.targets[0], node.value
         self.access = _access.OUTPUT
@@ -561,9 +548,16 @@ class _AnalyzeVisitor(ast.NodeVisitor, _ConversionMixin):
 
     def visit_Break(self, node):
         self.labelStack[-2].isActive = True
-        
+
     def visit_Call(self, node):
         self.visit(node.func)
+        f = self.getObj(node.func)
+        node.obj = None
+
+        if f is print:
+            self.visit_Print(node)
+            return
+
         self.access = _access.UNKNOWN
         for arg in node.args:
             self.visit(arg)
@@ -571,8 +565,6 @@ class _AnalyzeVisitor(ast.NodeVisitor, _ConversionMixin):
             self.visit(kw)
         self.access = _access.INPUT
         argsAreInputs = True
-        f = self.getObj(node.func)
-        node.obj = None
         if type(f) is type and issubclass(f, intbv):
             node.obj = self.getVal(node)
         elif f is concat:
@@ -582,7 +574,7 @@ class _AnalyzeVisitor(ast.NodeVisitor, _ConversionMixin):
             node.obj = int(0) # XXX
         elif f is bool:
             node.obj = bool()
-        elif f in (int, long, ord):
+        elif f in _flatten(integer_types, ord):
             node.obj = int(-1)
 ##         elif f in (posedge , negedge):
 ##             node.obj = _EdgeDetector()
@@ -597,26 +589,20 @@ class _AnalyzeVisitor(ast.NodeVisitor, _ConversionMixin):
             pass
         elif type(f) is FunctionType:
             argsAreInputs = False
-            s = inspect.getsource(f)
-            s = _dedent(s)
-            tree = ast.parse(s)
-            # print ast.dump(tree)
-            # print tree
+            tree = _makeAST(f)
             fname = f.__name__
             tree.name = _Label(fname)
-            tree.sourcefile = inspect.getsourcefile(f)
-            tree.lineoffset = inspect.getsourcelines(f)[1]-1
-            tree.symdict = f.func_globals.copy()
+            tree.symdict = f.__globals__.copy()
             tree.nonlocaldict = {}
             if fname in self.tree.callstack:
                 self.raiseError(node, _error.NotSupported, "Recursive call")
             tree.callstack = self.tree.callstack[:]
             tree.callstack.append(fname)
             # handle free variables
-            if f.func_code.co_freevars:
-                for n, c in zip(f.func_code.co_freevars, f.func_closure):
+            if f.__code__.co_freevars:
+                for n, c in zip(f.__code__.co_freevars, f.__closure__):
                     obj = _cell_deref(c)
-                    if not  isinstance(obj, (int, long, _Signal)):
+                    if not  isinstance(obj, (integer_types, _Signal)):
                         self.raiseError(node, _error.FreeVarTypeError, n)
                     tree.symdict[n] = obj
             v = _FirstPassVisitor(tree)
@@ -625,7 +611,7 @@ class _AnalyzeVisitor(ast.NodeVisitor, _ConversionMixin):
             v.visit(tree)
             node.obj = tree.returnObj
             node.tree = tree
-            tree.argnames = argnames = [arg.id for arg in tree.body[0].args.args]
+            tree.argnames = argnames = _get_argnames(tree.body[0])
             # extend argument list with keyword arguments on the correct position
             node.args.extend([None]*len(node.keywords))
             for kw in node.keywords:
@@ -659,8 +645,8 @@ class _AnalyzeVisitor(ast.NodeVisitor, _ConversionMixin):
             val = arg.obj
             if isinstance(val, bool):
                 val = int(val) # cast bool to int first
-            if isinstance(val, (EnumItemType, int, long)):
-                node.case = (node.left, val)         
+            if isinstance(val, (EnumItemType, integer_types)):
+                node.case = (node.left, val)
             # check whether it can be part of an edge check
             n = node.left.id
             if n in self.tree.sigdict:
@@ -685,7 +671,7 @@ class _AnalyzeVisitor(ast.NodeVisitor, _ConversionMixin):
 
     def visit_Str(self, node):
         node.obj = node.s
-           
+
     def visit_Continue(self, node):
         self.labelStack[-1].isActive = True
 
@@ -698,20 +684,20 @@ class _AnalyzeVisitor(ast.NodeVisitor, _ConversionMixin):
         self.visit(node.target)
         var = node.target.id
         self.tree.vardict[var] = int(-1)
-        
+
         cf = node.iter
         self.visit(cf)
         self.require(node, isinstance(cf, ast.Call), "Expected (down)range call")
         f = self.getObj(cf.func)
         self.require(node, f in (range, downrange), "Expected (down)range call")
-        
+
         for stmt in node.body:
             self.visit(stmt)
         self.refStack.pop()
         self.require(node, not node.orelse, "for-else not supported")
         self.labelStack.pop()
         self.labelStack.pop()
-        
+
     def visit_FunctionDef(self, node):
         raise AssertionError("subclass must implement this")
 
@@ -751,7 +737,7 @@ class _AnalyzeVisitor(ast.NodeVisitor, _ConversionMixin):
         node.isCase = True
         node.caseVar = var1
         node.caseItem = item1
-        if (len(choices) == _getNritems(var1.obj)) or node.else_:
+        if node.else_ or (len(choices) == _getNritems(var1.obj)) :
             node.isFullCase = True
 
     def visit_ListComp(self, node):
@@ -759,7 +745,7 @@ class _AnalyzeVisitor(ast.NodeVisitor, _ConversionMixin):
         self.kind = _kind.DECLARATION
         try:
             self.visit(node.elt)
-        except ConversionError, e:
+        except ConversionError as e:
             if e.kind == _error.UnboundLocal:
                 pass
             else:
@@ -777,6 +763,9 @@ class _AnalyzeVisitor(ast.NodeVisitor, _ConversionMixin):
             self.raiseError(node, _error.UnsupportedListComp)
         mem.depth = cf.args[0].obj
 
+    def visit_NameConstant(self, node):
+        node.obj = node.value
+
     def visit_Name(self, node):
         if isinstance(node.ctx, ast.Store):
             self.setName(node)
@@ -819,9 +808,12 @@ class _AnalyzeVisitor(ast.NodeVisitor, _ConversionMixin):
             # mark shadow signal as driven only when they are seen somewhere
             if isinstance(sig, _ShadowSignal):
                 sig._driven = 'wire'
+            # mark tristate signal as driven if its driver is seen somewhere
+            if isinstance(sig, _TristateDriver):
+                sig._sig._driven = 'wire'
             if not isinstance(sig, _Signal):
                 # print "not a signal: %s" % n
-                pass 
+                pass
             else:
                 if sig._type is bool:
                     node.edge = sig.posedge
@@ -840,7 +832,7 @@ class _AnalyzeVisitor(ast.NodeVisitor, _ConversionMixin):
                 self.tree.outputs.add(n)
             elif self.access == _access.UNKNOWN:
                 pass
-            else: 
+            else:
                 self.raiseError(node, _error.NotSupported, "Augmented signal assignment")
         if n in self.tree.vardict:
             obj = self.tree.vardict[n]
@@ -888,11 +880,11 @@ class _AnalyzeVisitor(ast.NodeVisitor, _ConversionMixin):
                     _constDict[ws] = self.tree.symdict[ws]
                     if ext:
                         _extConstDict[ws] = self.tree.symdict[ws]
-        elif n in __builtin__.__dict__:
-            node.obj = __builtin__.__dict__[n]
+        elif n in builtins.__dict__:
+            node.obj = builtins.__dict__[n]
         else:
             self.raiseError(node, _error.UnboundLocal, n)
-        
+
     def visit_Return(self, node):
         self.raiseError(node, _error.NotSupported, "return statement")
 
@@ -901,7 +893,13 @@ class _AnalyzeVisitor(ast.NodeVisitor, _ConversionMixin):
         f = []
         nr = 0
         a = []
-        for n in node.values:
+
+        if PY2 and isinstance(node, ast.Print):
+            node_args = node.values
+        else:
+            node_args = node.args
+
+        for n in node_args:
             if isinstance(n, ast.BinOp) and isinstance(n.op, ast.Mod) and \
                isinstance(n.left, ast.Str):
                 if isinstance(n.right, ast.Tuple):
@@ -950,7 +948,7 @@ class _AnalyzeVisitor(ast.NodeVisitor, _ConversionMixin):
         if len(node.args) > nr:
             self.raiseError(node, _error.FormatString, "too many arguments")
         self.generic_visit(node)
-        
+
     def visit_Subscript(self, node):
         if isinstance(node.slice, ast.Slice):
             self.accessSlice(node)
@@ -975,7 +973,7 @@ class _AnalyzeVisitor(ast.NodeVisitor, _ConversionMixin):
                 else:
                     rightind = 0
                 node.obj = node.obj[leftind:rightind]
-    
+
     def accessIndex(self, node):
         self.visit(node.value)
         self.access = _access.INPUT
@@ -1038,15 +1036,15 @@ class _AnalyzeVisitor(ast.NodeVisitor, _ConversionMixin):
             self.raiseError(node, _error.UnsupportedYield)
         node.senslist = senslist
 
-        
+
 class _AnalyzeBlockVisitor(_AnalyzeVisitor):
-    
+
     def __init__(self, tree):
         _AnalyzeVisitor.__init__(self, tree)
         for n, v in self.tree.symdict.items():
             if isinstance(v, _Signal):
                 self.tree.sigdict[n] = v
-        
+
 
     def visit_FunctionDef(self, node):
         self.refStack.push()
@@ -1063,7 +1061,7 @@ class _AnalyzeBlockVisitor(_AnalyzeVisitor):
                 self.tree.kind = _kind.INITIAL
         self.refStack.pop()
 
-                
+
     def visit_Module(self, node):
         self.generic_visit(node)
         for n in self.tree.outputs:
@@ -1074,8 +1072,8 @@ class _AnalyzeBlockVisitor(_AnalyzeVisitor):
         for n in self.tree.inputs:
             s = self.tree.sigdict[n]
             s._markRead()
-            
-     
+
+
     def visit_Return(self, node):
         ### value should be None
         if node.value is None:
@@ -1089,7 +1087,7 @@ class _AnalyzeBlockVisitor(_AnalyzeVisitor):
 
 
 class _AnalyzeAlwaysCombVisitor(_AnalyzeBlockVisitor):
-    
+
     def __init__(self, tree, senslist):
         _AnalyzeBlockVisitor.__init__(self, tree)
         self.tree.senslist = senslist
@@ -1129,10 +1127,10 @@ class _AnalyzeAlwaysCombVisitor(_AnalyzeBlockVisitor):
             for n in self.tree.outmems:
                 m = _getMemInfo(self.tree.symdict[n])
                 m._driven = "wire"
-                
-                
+
+
 class _AnalyzeAlwaysSeqVisitor(_AnalyzeBlockVisitor):
-    
+
     def __init__(self, tree, senslist, reset, sigregs, varregs):
         _AnalyzeBlockVisitor.__init__(self, tree)
         self.tree.senslist = senslist
@@ -1140,22 +1138,22 @@ class _AnalyzeAlwaysSeqVisitor(_AnalyzeBlockVisitor):
         self.tree.sigregs = sigregs
         self.tree.varregs = varregs
 
-         
+
     def visit_FunctionDef(self, node):
         self.refStack.push()
         for n in node.body:
             self.visit(n)
         self.tree.kind = _kind.ALWAYS_SEQ
         self.refStack.pop()
-     
+
 
 class _AnalyzeAlwaysDecoVisitor(_AnalyzeBlockVisitor):
-    
+
     def __init__(self, tree, senslist):
         _AnalyzeBlockVisitor.__init__(self, tree)
         self.tree.senslist = senslist
 
-         
+
     def visit_FunctionDef(self, node):
         self.refStack.push()
         for n in node.body:
@@ -1163,10 +1161,10 @@ class _AnalyzeAlwaysDecoVisitor(_AnalyzeBlockVisitor):
         self.tree.kind = _kind.ALWAYS_DECO
         self.refStack.pop()
 
-            
+
 
 class _AnalyzeFuncVisitor(_AnalyzeVisitor):
-    
+
     def __init__(self, tree, args, keywords):
         _AnalyzeVisitor.__init__(self, tree)
         self.args = args
@@ -1174,10 +1172,10 @@ class _AnalyzeFuncVisitor(_AnalyzeVisitor):
         self.tree.hasReturn = False
         self.tree.returnObj = None
 
-        
+
     def visit_FunctionDef(self, node):
         self.refStack.push()
-        argnames = [arg.id for arg in node.args.args]
+        argnames = _get_argnames(node)
         for i, arg in enumerate(self.args):
             n = argnames[i]
             self.tree.symdict[n] = self.getObj(arg)
@@ -1241,10 +1239,32 @@ ismethod = inspect.ismethod
 def isboundmethod(m):
     return ismethod(m) and m.__self__ is not None
 
-def _analyzeTopFunc(func, *args, **kwargs):
+
+def _analyzeTopFunc(top_inst, func, *args, **kwargs):
     tree = _makeAST(func)
     v = _AnalyzeTopFuncVisitor(func, tree, *args, **kwargs)
     v.visit(tree)
+
+    objs = []
+    for name, obj in v.fullargdict.items():
+        if not isinstance(obj, _Signal):
+            objs.append((name, obj))
+
+    #create ports for any signal in the top instance if it was buried in an
+    #object passed as in argument
+    #TODO: This will not work for nested objects in the top level
+    for name, obj in objs:
+        if not hasattr(obj, '__dict__'):
+            continue
+        for attr, attrobj in vars(obj).items():
+            if isinstance(attrobj, _Signal):
+                signame = attrobj._name
+                if not signame:
+                    signame = name + '_' + attr
+                    attrobj._name = signame
+                v.argdict[signame] = attrobj
+                v.argnames.append(signame)
+
     return v
 
 class _AnalyzeTopFuncVisitor(_AnalyzeVisitor):
@@ -1255,30 +1275,34 @@ class _AnalyzeTopFuncVisitor(_AnalyzeVisitor):
         self.args = args
         self.kwargs = kwargs
         self.name = None
+        self.fullargdict = {}
         self.argdict = {}
+        self.argnames = []
 
     def visit_FunctionDef(self, node):
-        
+
         self.name = node.name
-        argnames = [arg.id for arg in node.args.args]
+        self.argnames = _get_argnames(node)
         if isboundmethod(self.func):
-            if not argnames[0] == 'self':
+            if not self.argnames[0] == 'self':
                 self.raiseError(node, _error.NotSupported,
                                 "first method argument name other than 'self'")
             # skip self
-            argnames = argnames[1:]  
+            self.argnames = self.argnames[1:]
         i=-1
         for i, arg in enumerate(self.args):
-            n = argnames[i]
+            n = self.argnames[i]
+            self.fullargdict[n] = arg
             if isinstance(arg, _Signal):
-                self.argdict[n] = arg  
+                self.argdict[n] = arg
             if _isMem(arg):
                 self.raiseError(node, _error.ListAsPort, n)
-        for n in argnames[i+1:]:
+        for n in self.argnames[i+1:]:
             if n in self.kwargs:
                 arg = self.kwargs[n]
+                self.fullargdict[n] = arg
                 if isinstance(arg, _Signal):
                     self.argdict[n] = arg
                 if _isMem(arg):
                     self.raiseError(node, _error.ListAsPort, n)
-        self.argnames = [n for n in argnames if n in self.argdict]
+        self.argnames = [n for n in self.argnames if n in self.argdict]
