@@ -20,32 +20,38 @@
 """ myhdl toVerilog conversion module.
 
 """
+from __future__ import absolute_import
+from __future__ import print_function
 
 
 import sys
 import math
+import os
+
 import inspect
 from datetime import datetime
 import ast
 import string
 
-from types import GeneratorType, ClassType, TypeType
-from cStringIO import StringIO
+from types import GeneratorType
+from myhdl._compat import StringIO
 import warnings
 
 import myhdl
 from myhdl import *
+from myhdl._compat import integer_types, class_types, PY2
 from myhdl import ToVerilogError, ToVerilogWarning
 from myhdl._extractHierarchy import (_HierExtr, _isMem, _getMemInfo,
                                      _UserVerilogCode, _userCodeMap)
 
 from myhdl._instance import _Instantiator
-from myhdl.conversion._misc import (_error, _kind, _context, 
+from myhdl.conversion._misc import (_error, _kind, _context,
                                     _ConversionMixin, _Label, _genUniqueSuffix, _isConstant)
-from myhdl.conversion._analyze import (_analyzeSigs, _analyzeGens, _analyzeTopFunc, 
+from myhdl.conversion._analyze import (_analyzeSigs, _analyzeGens, _analyzeTopFunc,
                                        _Ram, _Rom)
 from myhdl._Signal import _Signal
-            
+
+
 _converting = 0
 _profileFunc = None
 
@@ -53,7 +59,7 @@ def _checkArgs(arglist):
     for arg in arglist:
         if not isinstance(arg, (GeneratorType, _Instantiator, _UserVerilogCode)):
             raise ToVerilogError(_error.ArgType, arg)
-        
+
 def _flatten(*args):
     arglist = []
     for arg in args:
@@ -79,18 +85,22 @@ def _makeDoc(doc, indent=''):
 
 class _ToVerilogConvertor(object):
 
-    __slots__ = ("name", 
-                 "timescale", 
+    __slots__ = ("name",
+                 "directory",
+                 "timescale",
                  "standard",
-                 "prefer_blocking_assignments", 
+                 "prefer_blocking_assignments",
                  "radix",
                  "header",
                  "no_myhdl_header",
-                 "no_testbench"
+                 "no_testbench",
+                 "portmap",
+                 "trace"
                  )
 
     def __init__(self):
         self.name = None
+        self.directory = None
         self.timescale = "1ns/10ps"
         self.standard = '2001'
         self.prefer_blocking_assignments = True
@@ -98,6 +108,7 @@ class _ToVerilogConvertor(object):
         self.header = ''
         self.no_myhdl_header = False
         self.no_testbench = False
+        self.trace = False
 
     def __call__(self, func, *args, **kwargs):
         global _converting
@@ -114,7 +125,7 @@ class _ToVerilogConvertor(object):
 
         _converting = 1
         if self.name is None:
-            name = func.func_name
+            name = func.__name__
         else:
             name = str(self.name)
         try:
@@ -122,22 +133,29 @@ class _ToVerilogConvertor(object):
         finally:
             _converting = 0
 
-        vpath = name + ".v"
+        if self.directory is None:
+            directory = ''
+        else:
+            directory = self.directory
+
+        vfilename = name + ".v"
+        vpath = os.path.join(directory, vfilename)
         vfile = open(vpath, 'w')
-        
+
         ### initialize properly ###
         _genUniqueSuffix.reset()
 
-        siglist, memlist = _analyzeSigs(h.hierarchy)
         arglist = _flatten(h.top)
         # print h.top
         _checkArgs(arglist)
         genlist = _analyzeGens(arglist, h.absnames)
+        siglist, memlist = _analyzeSigs(h.hierarchy)
         _annotateTypes(genlist)
-        intf = _analyzeTopFunc(func, *args, **kwargs)
+        top_inst = h.hierarchy[0]
+        intf = _analyzeTopFunc(top_inst, func, *args, **kwargs)
         intf.name = name
         doc = _makeDoc(inspect.getdoc(func))
-        
+
         self._convert_filter(h, intf, siglist, memlist, genlist)
 
         _writeFileHeader(vfile, vpath, self.timescale)
@@ -150,16 +168,23 @@ class _ToVerilogConvertor(object):
 
         # don't write testbench if module has no ports
         if len(intf.argnames) > 0 and not toVerilog.no_testbench:
-            tbpath = "tb_" + vpath
+            tbpath = os.path.join(directory, "tb_" + vfilename)
             tbfile = open(tbpath, 'w')
-            _writeTestBench(tbfile, intf)
+            _writeTestBench(tbfile, intf, self.trace)
             tbfile.close()
 
+        # build portmap for cosimulation
+        portmap = {}
+        for n, s in intf.argdict.items():
+            if hasattr(s, 'driver'): portmap[n] = s.driver()
+            else: portmap[n] = s
+        self.portmap = portmap
+
         ### clean-up properly ###
         self._cleanup(siglist)
-        
+
         return h.top
-        
+
     def _cleanup(self, siglist):
         # clean up signal names
         for sig in siglist:
@@ -167,7 +192,7 @@ class _ToVerilogConvertor(object):
 #             sig._name = None
 #             sig._driven = False
 #             sig._read = False
-            
+
         # clean up attributes
         self.name = None
         self.standard = '2001'
@@ -176,13 +201,14 @@ class _ToVerilogConvertor(object):
         self.header = ""
         self.no_myhdl_header = False
         self.no_testbench = False
-        
-        
+        self.trace = False
+
+
     def _convert_filter(self, h, intf, siglist, memlist, genlist):
-        # intended to be a entry point for other uses: 
+        # intended to be a entry point for other uses:
         #  code checking, optimizations, etc
         pass
-    
+
 
 toVerilog = _ToVerilogConvertor()
 
@@ -193,29 +219,29 @@ myhdl_header = """\
 """
 
 def _writeFileHeader(f, fn, ts):
-    vars = dict(filename=fn, 
+    vars = dict(filename=fn,
                 version=myhdl.__version__,
                 date=datetime.today().ctime()
                 )
     if not toVerilog.no_myhdl_header:
-        print >> f, string.Template(myhdl_header).substitute(vars)
+        print(string.Template(myhdl_header).substitute(vars), file=f)
     if toVerilog.header:
-        print >> f, string.Template(toVerilog.header).substitute(vars)
-    print >> f
-    print >> f, "`timescale %s" % ts
-    print >> f
+        print(string.Template(toVerilog.header).substitute(vars), file=f)
+    print(file=f)
+    print("`timescale %s" % ts, file=f)
+    print(file=f)
 
 
 def _writeModuleHeader(f, intf, doc):
-    print >> f, "module %s (" % intf.name
+    print("module %s (" % intf.name, file=f)
     b = StringIO()
     for portname in intf.argnames:
-        print >> b, "    %s," % portname
-    print >> f, b.getvalue()[:-2]
+        print("    %s," % portname, file=b)
+    print(b.getvalue()[:-2], file=f)
     b.close()
-    print >> f, ");"
-    print >> f, doc
-    print >> f
+    print(");", file=f)
+    print(doc, file=f)
+    print(file=f)
     for portname in intf.argnames:
         s = intf.argdict[portname]
         if s._name is None:
@@ -231,18 +257,18 @@ def _writeModuleHeader(f, intf, doc):
                 warnings.warn("%s: %s" % (_error.OutputPortRead, portname),
                               category=ToVerilogWarning
                               )
-            print >> f, "output %s%s%s;" % (p, r, portname)
+            print("output %s%s%s;" % (p, r, portname), file=f)
             if s._driven == 'reg':
-                print >> f, "reg %s%s%s;" % (p, r, portname)
+                print("reg %s%s%s;" % (p, r, portname), file=f)
             else:
-                print >> f, "wire %s%s%s;" % (p, r, portname)
+                print("wire %s%s%s;" % (p, r, portname), file=f)
         else:
             if not s._read:
                 warnings.warn("%s: %s" % (_error.UnusedPort, portname),
                               category=ToVerilogWarning
                               )
-            print >> f, "input %s%s%s;" % (p, r, portname)
-    print >> f
+            print("input %s%s%s;" % (p, r, portname), file=f)
+    print(file=f)
 
 
 def _writeSigDecls(f, intf, siglist, memlist):
@@ -264,7 +290,7 @@ def _writeSigDecls(f, intf, siglist, memlist):
                 k = 'reg'
             # the following line implements initial value assignments
             # print >> f, "%s %s%s = %s;" % (k, r, s._name, int(s._val))
-            print >> f, "%s %s%s%s;" % (k, p, r, s._name)
+            print("%s %s%s%s;" % (k, p, r, s._name), file=f)
         elif s._read:
             # the original exception
             # raise ToVerilogError(_error.UndrivenSignal, s._name)
@@ -273,8 +299,8 @@ def _writeSigDecls(f, intf, siglist, memlist):
                           category=ToVerilogWarning
                           )
             constwires.append(s)
-            print >> f, "wire %s%s;" % (r, s._name)
-    print >> f
+            print("wire %s%s;" % (r, s._name), file=f)
+    print(file=f)
     for m in memlist:
         if not m._used:
             continue
@@ -290,30 +316,30 @@ def _writeSigDecls(f, intf, siglist, memlist):
         p = _getSignString(m.elObj)
         k = 'wire'
         if m._driven:
-            k = m._driven 
-        print >> f, "%s %s%s%s [0:%s-1];" % (k, p, r, m.name, m.depth)
-    print >> f
+            k = m._driven
+        print("%s %s%s%s [0:%s-1];" % (k, p, r, m.name, m.depth), file=f)
+    print(file=f)
     for s in constwires:
         if s._type in (bool, intbv):
             c = int(s.val)
         else:
             raise ToVerilogError("Unexpected type for constant signal", s._name)
-        print >> f, "assign %s = %s;" % (s._name, c)
-    print >> f
+        print("assign %s = %s;" % (s._name, c), file=f)
+    print(file=f)
     # shadow signal assignments
     for s in siglist:
-        if hasattr(s, 'toVerilog') and s._read:
-            print >> f, s.toVerilog()
-    print >> f
+        if hasattr(s, 'toVerilog') and s._driven:
+            print(s.toVerilog(), file=f)
+    print(file=f)
 
 
 def _writeModuleFooter(f):
-    print >> f, "endmodule"
-    
+    print("endmodule", file=f)
 
-def _writeTestBench(f, intf):
-    print >> f, "module tb_%s;" % intf.name
-    print >> f
+
+def _writeTestBench(f, intf, trace=False):
+    print("module tb_%s;" % intf.name, file=f)
+    print(file=f)
     fr = StringIO()
     to = StringIO()
     pm = StringIO()
@@ -321,29 +347,32 @@ def _writeTestBench(f, intf):
         s = intf.argdict[portname]
         r = _getRangeString(s)
         if s._driven:
-            print >> f, "wire %s%s;" % (r, portname)
-            print >> to, "        %s," % portname
+            print("wire %s%s;" % (r, portname), file=f)
+            print("        %s," % portname, file=to)
         else:
-            print >> f, "reg %s%s;" % (r, portname)
-            print >> fr, "        %s," % portname
-        print >> pm, "    %s," % portname
-    print >> f
-    print >> f, "initial begin"
+            print("reg %s%s;" % (r, portname), file=f)
+            print("        %s," % portname, file=fr)
+        print("    %s," % portname, file=pm)
+    print(file=f)
+    print("initial begin", file=f)
+    if trace:
+        print('    $dumpfile("%s.vcd");' % intf.name, file=f)
+        print('    $dumpvars(0, dut);', file=f)
     if fr.getvalue():
-        print >> f, "    $from_myhdl("
-        print >> f, fr.getvalue()[:-2]
-        print >> f, "    );"
+        print("    $from_myhdl(", file=f)
+        print(fr.getvalue()[:-2], file=f)
+        print("    );", file=f)
     if to.getvalue():
-        print >> f, "    $to_myhdl("
-        print >> f, to.getvalue()[:-2]
-        print >> f, "    );" 
-    print >> f, "end"
-    print >> f
-    print >> f, "%s dut(" % intf.name
-    print >> f, pm.getvalue()[:-2]
-    print >> f, ");"
-    print >> f
-    print >> f, "endmodule"
+        print("    $to_myhdl(", file=f)
+        print(to.getvalue()[:-2], file=f)
+        print("    );", file=f)
+    print("end", file=f)
+    print(file=f)
+    print("%s dut(" % intf.name, file=f)
+    print(pm.getvalue()[:-2], file=f)
+    print(");", file=f)
+    print(file=f)
+    print("endmodule", file=f)
 
 
 def _getRangeString(s):
@@ -378,7 +407,7 @@ def _convertGens(genlist, vfile):
         elif tree.kind == _kind.ALWAYS_DECO:
             Visitor = _ConvertAlwaysDecoVisitor
         elif tree.kind == _kind.ALWAYS_SEQ:
-            Visitor = _ConvertAlwaysSeqVisitor    
+            Visitor = _ConvertAlwaysSeqVisitor
         else: # ALWAYS_COMB
             Visitor = _ConvertAlwaysCombVisitor
         v = Visitor(tree, blockBuf, funcBuf)
@@ -414,10 +443,16 @@ opmap = {
     ast.Or       : '||',
 }
 
+nameconstant_map = {
+    True: "1'b1",
+    False: "1'b0",
+    None: "'bz"
+}
+
 
 
 class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
-    
+
     def __init__(self, tree, buf):
         self.tree = tree
         self.buf = buf
@@ -440,13 +475,13 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
     def writeline(self, nr=1):
         for i in range(nr):
             self.buf.write("\n%s" % self.ind)
-            
+
     def writeDoc(self, node):
         assert hasattr(node, 'doc')
         doc = _makeDoc(node.doc, self.ind)
         self.write(doc)
         self.writeline()
-            
+
     def indent(self):
         self.ind += ' ' * 4
 
@@ -502,7 +537,7 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
             self.write(" = %s;" % inival)
         else:
             self.write(";")
-        
+
     def writeDeclarations(self):
         for name, obj in self.tree.vardict.items():
             self.writeline()
@@ -566,7 +601,7 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
             self.write(" %s " % opmap[type(node.op)])
             self.visit(n)
         self.write(")")
-        
+
     def visit_UnaryOp(self, node):
         self.write("(%s" % opmap[type(node.op)])
         self.visit(node.operand)
@@ -584,6 +619,10 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
         self.visit(node.value)
 
     def getAttr(self, node):
+        if isinstance(node.value, ast.Subscript):
+            self.setAttr(node)
+            return
+
         assert isinstance(node.value, ast.Name), node.value
         n = node.value.id
         if n in self.tree.symdict:
@@ -683,6 +722,11 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
         fn = node.func
         # assert isinstance(fn, astNode.Name)
         f = self.getObj(fn)
+
+        if f is print:
+            self.visit_Print(node)
+            return
+
         opening, closing = '(', ')'
         if f is bool:
             self.write("(")
@@ -693,7 +737,7 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
         elif f is len:
             val = self.getVal(node)
             self.require(node, val is not None, "cannot calculate len")
-            self.write(`val`)
+            self.write(repr(val))
             return
         elif f is now:
             self.write("$time")
@@ -705,7 +749,7 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
                     self.raiseError(node, _error.UnsupportedType, "Strings with length > 1")
                 else:
                     node.args[0].s = str(ord(node.args[0].s))
-        elif f in (int, long):
+        elif f in integer_types:
             opening, closing = '', ''
             # convert number argument to integer
             if isinstance(node.args[0], ast.Num):
@@ -721,7 +765,7 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
             self.write(opening)
             self.visit(fn.value)
             self.write(closing)
-        elif type(f) in (ClassType, TypeType) and issubclass(f, Exception):
+        elif (type(f) in class_types) and issubclass(f, Exception):
             self.write(f.__name__)
         elif f in (posedge, negedge):
             opening, closing = ' ', ''
@@ -793,7 +837,7 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
         # ugly hack to detect an orphan "task" call
         if isinstance(expr, ast.Call) and hasattr(expr, 'tree'):
             self.write(';')
-                       
+
     def visit_IfExp(self, node):
         self.visit(node.test)
         self.write(' ? ')
@@ -912,7 +956,7 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
         self.dedent()
         self.writeline()
         self.write("endcase")
-        
+
     def mapToIf(self, node, *args):
         first = True
         for test, suite in node.tests:
@@ -945,10 +989,13 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
     def visit_Module(self, node, *args):
         for stmt in node.body:
             self.visit(stmt)
-            
+
     def visit_ListComp(self, node):
         pass # do nothing
 
+    def visit_NameConstant(self, node):
+        self.write(nameconstant_map[node.obj])
+
     def visit_Name(self, node):
         if isinstance(node.ctx, ast.Store):
             self.setName(node)
@@ -959,16 +1006,14 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
         self.write(node.id)
 
     def getName(self, node):
+        n = node.id
+        if PY2 and n in ('True', 'False', 'None'):
+            self.visit_NameConstant(node)
+            return
+
         addSignBit = False
         isMixedExpr = (not node.signed) and (self.context == _context.SIGNED)
-        n = node.id
-        if n == 'False':
-            s = "1'b0"
-        elif n == 'True':
-            s = "1'b1"
-        elif n == 'None':
-            s = "'bz"
-        elif n in self.tree.vardict:
+        if n in self.tree.vardict:
             addSignBit = isMixedExpr
             s = n
         elif n in self.tree.argnames:
@@ -979,7 +1024,7 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
             obj = self.tree.symdict[n]
             if isinstance(obj, bool):
                 s = "%s" % int(obj)
-            elif isinstance(obj, (int, long)):
+            elif isinstance(obj, integer_types):
                 s = self.IntRepr(obj)
             elif isinstance(obj, _Signal):
                 addSignBit = isMixedExpr
@@ -990,7 +1035,7 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
                 s = m.name
             elif isinstance(obj, EnumItemType):
                 s = obj._toVerilog()
-            elif type(obj) in (ClassType, TypeType) and issubclass(obj, Exception):
+            elif (type(obj) in class_types) and issubclass(obj, Exception):
                 s = n
             else:
                 self.raiseError(node, _error.UnsupportedType, "%s, %s" % (n, type(obj)))
@@ -1000,7 +1045,7 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
             self.write("$signed({1'b0, ")
         self.write(s)
         if addSignBit:
-            self.write("})")       
+            self.write("})")
 
     def visit_Pass(self, node):
         self.write("// pass")
@@ -1054,7 +1099,7 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
 
     def visit_Raise(self, node):
         self.write("$finish;")
-    
+
     def visit_Return(self, node):
         self.write("disable %s;" % self.returnLabel)
 
@@ -1163,9 +1208,9 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
             self.writeSensitivityList(senslist)
             self.write(";")
 
-        
+
 class _ConvertAlwaysVisitor(_ConvertVisitor):
-    
+
     def __init__(self, tree, blockBuf, funcBuf):
         _ConvertVisitor.__init__(self, tree, blockBuf)
         self.funcBuf = funcBuf
@@ -1190,14 +1235,14 @@ class _ConvertAlwaysVisitor(_ConvertVisitor):
 
 
 class _ConvertInitialVisitor(_ConvertVisitor):
-    
+
     def __init__(self, tree, blockBuf, funcBuf):
         _ConvertVisitor.__init__(self, tree, blockBuf)
         self.funcBuf = funcBuf
 
     def visit_FunctionDef(self, node):
         self.writeDoc(node)
-        self.write("initial begin: %s" % self.tree.name) 
+        self.write("initial begin: %s" % self.tree.name)
         self.indent()
         self.writeDeclarations()
         self.visit_stmt(node.body)
@@ -1209,7 +1254,7 @@ class _ConvertInitialVisitor(_ConvertVisitor):
 
 
 class _ConvertAlwaysCombVisitor(_ConvertVisitor):
-    
+
     def __init__(self, tree, blockBuf, funcBuf):
         _ConvertVisitor.__init__(self, tree, blockBuf)
         if toVerilog.prefer_blocking_assignments:
@@ -1227,9 +1272,9 @@ class _ConvertAlwaysCombVisitor(_ConvertVisitor):
         self.writeline(2)
 
 
-        
+
 class _ConvertSimpleAlwaysCombVisitor(_ConvertVisitor):
-    
+
     def __init__(self, tree, blockBuf, funcBuf):
         _ConvertVisitor.__init__(self, tree, blockBuf)
         self.funcBuf = funcBuf
@@ -1245,9 +1290,9 @@ class _ConvertSimpleAlwaysCombVisitor(_ConvertVisitor):
         self.visit_stmt(node.body)
         self.writeline(2)
 
-     
+
 class _ConvertAlwaysDecoVisitor(_ConvertVisitor):
-    
+
     def __init__(self, tree, blockBuf, funcBuf):
         _ConvertVisitor.__init__(self, tree, blockBuf)
         self.funcBuf = funcBuf
@@ -1277,10 +1322,10 @@ def _convertInitVal(reg, init):
         assert isinstance(init, EnumItemType)
         v = init._toVerilog()
     return v
- 
+
 
 class _ConvertAlwaysSeqVisitor(_ConvertVisitor):
-    
+
     def __init__(self, tree, blockBuf, funcBuf):
         _ConvertVisitor.__init__(self, tree, blockBuf)
         self.funcBuf = funcBuf
@@ -1319,9 +1364,9 @@ class _ConvertAlwaysSeqVisitor(_ConvertVisitor):
         self.write("end")
         self.writeline(2)
 
-    
+
 class _ConvertFunctionVisitor(_ConvertVisitor):
-    
+
     def __init__(self, tree, funcBuf):
         _ConvertVisitor.__init__(self, tree, funcBuf)
         self.returnObj = tree.returnObj
@@ -1336,7 +1381,7 @@ class _ConvertFunctionVisitor(_ConvertVisitor):
             obj = self.tree.symdict[name]
             self.writeline()
             self.writeDeclaration(obj, name, "input")
-            
+
     def visit_FunctionDef(self, node):
         self.write("function ")
         self.writeOutputDeclaration()
@@ -1364,9 +1409,9 @@ class _ConvertFunctionVisitor(_ConvertVisitor):
 
 
 
-    
+
 class _ConvertTaskVisitor(_ConvertVisitor):
-    
+
     def __init__(self, tree, funcBuf):
         _ConvertVisitor.__init__(self, tree, funcBuf)
         self.returnLabel = _Label("RETURN")
@@ -1380,7 +1425,7 @@ class _ConvertTaskVisitor(_ConvertVisitor):
             dir = (inout and "inout") or (output and "output") or "input"
             self.writeline()
             self.writeDeclaration(obj, name, dir)
-            
+
     def visit_FunctionDef(self, node):
         self.write("task %s;" % self.tree.name)
         self.indent()
@@ -1402,7 +1447,7 @@ class _ConvertTaskVisitor(_ConvertVisitor):
 def _maybeNegative(obj):
     if hasattr(obj, '_min') and (obj._min is not None) and (obj._min < 0):
         return True
-    if isinstance(obj, (int, long)) and obj < 0:
+    if isinstance(obj, integer_types) and obj < 0:
         return True
     return False
 
@@ -1410,12 +1455,12 @@ class _AnnotateTypesVisitor(ast.NodeVisitor, _ConversionMixin):
 
     def __init__(self, tree):
         self.tree = tree
-        
+
     def visit_FunctionDef(self, node):
         # don't visit arguments and decorators
         for stmt in node.body:
-            self.visit(stmt)   
-        
+            self.visit(stmt)
+
     def visit_BinOp(self, node):
         self.visit(node.left)
         self.visit(node.right)
@@ -1423,7 +1468,7 @@ class _AnnotateTypesVisitor(ast.NodeVisitor, _ConversionMixin):
         # special treatement of subtraction unless in a top-level rhs
         if isinstance(node.op, ast.Sub) and not hasattr(node, 'isRhs'):
             node.signed = True
-      
+
     def visit_BoolOp(self, node):
         for n in node.values:
             self.visit(n)
@@ -1436,20 +1481,20 @@ class _AnnotateTypesVisitor(ast.NodeVisitor, _ConversionMixin):
             node.obj = int(-1)
             if isinstance(node.operand, ast.Num):
                 node.signed = True
-    
+
     def visit_Attribute(self, node):
         if isinstance(node.ctx, ast.Store):
             self.setAttr(node)
         else:
             self.getAttr(node)
-         
+
     def setAttr(self, node):
         self.visit(node.value)
 
     def getAttr(self, node):
         node.signed = False
         self.visit(node.value)
-                  
+
     def visit_Call(self, node):
         self.generic_visit(node)
         f = self.getObj(node.func)
@@ -1460,7 +1505,7 @@ class _AnnotateTypesVisitor(ast.NodeVisitor, _ConversionMixin):
         elif hasattr(node, 'tree'):
             v = _AnnotateTypesVisitor(node.tree)
             v.visit(node.tree)
-            node.signed = _maybeNegative(node.tree.returnObj)  
+            node.signed = _maybeNegative(node.tree.returnObj)
 
     def visit_Compare(self, node):
         node.signed = False
@@ -1469,7 +1514,7 @@ class _AnnotateTypesVisitor(ast.NodeVisitor, _ConversionMixin):
             self.visit(n)
             if n.signed:
                 node.signed = True
-                
+
     def visit_If(self, node):
         if node.ignore:
             return
@@ -1512,13 +1557,13 @@ class _AnnotateTypesVisitor(ast.NodeVisitor, _ConversionMixin):
         node.signed = False
         self.generic_visit(node)
 
-        
+
 def _annotateTypes(genlist):
     for tree in genlist:
         if isinstance(tree, _UserVerilogCode):
             continue
         v = _AnnotateTypesVisitor(tree)
         v.visit(tree)
-        
+
 
 
