@@ -5,9 +5,9 @@ from inc import Inc
 
 def GrayInc(graycnt, enable, clock, reset, width):
     
-    bincnt = Signal(intbv(0)[width:])
+    bincnt = Signal(modbv(0)[width:])
     
-    inc_1 = Inc(bincnt, enable, clock, reset, n=2**width)
+    inc_1 = Inc(bincnt, enable, clock, reset)
     bin2gray_1 = bin2gray(B=bincnt, G=graycnt, width=width)
     
     return inc_1, bin2gray_1
@@ -15,7 +15,7 @@ def GrayInc(graycnt, enable, clock, reset, width):
 
 def GrayIncReg(graycnt, enable, clock, reset, width):
     
-    graycnt_comb = Signal(intbv(0)[width:])
+    graycnt_comb = Signal(modbv(0)[width:])
     
     gray_inc_1 = GrayInc(graycnt_comb, enable, clock, reset, width)
 
@@ -28,8 +28,11 @@ def GrayIncReg(graycnt, enable, clock, reset, width):
 
 def main():
     width = 8
-    graycnt = Signal(intbv(0)[width:])
-    enable, clock, reset = [Signal(bool()) for i in range(3)]
+    graycnt = Signal(modbv(0)[width:])
+    enable = Signal(bool())
+    clock = Signal(bool())
+    reset = ResetSignal(0, active=0, async=True)
+
     toVerilog(GrayIncReg, graycnt, enable, clock, reset, width)
     toVHDL(GrayIncReg, graycnt, enable, clock, reset, width)
 
